@@ -16,6 +16,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import difflib
+import ipaddress
+import json
 from collections import OrderedDict
 
 
@@ -59,10 +62,20 @@ class Conf:
         else:
             raise ValueError('cannot update %s on finalized Conf object' % name)
 
-    def set_defaults(self):
+    def _set_default(self, key, value, conf=None):
+        if conf is None:
+            conf = self.__dict__
+        assert key in conf, key
+        if conf[key] is None:
+            conf[key] = value
+
+    def _set_conf_defaults(self, defaults, conf):
+        for key, value in defaults.items():
+            self._set_default(key, value, conf=conf)
+
+    def set_defaults(self, defaults=None, conf=None):
         """Set default values and run any basic sanity checks."""
-        for key, value in self.defaults.items():
-            self._set_default(key, value)
+        self._set_conf_defaults(self.defaults, self.__dict__)
 
     def _check_unknown_conf(self, conf):
         """Check that supplied conf dict doesn't specify keys not defined."""
@@ -125,15 +138,34 @@ class Conf:
         self.__dict__.update(
             {k: v for k, v in self._conf_keys(other_conf, dyn=True)})
 
-    def _set_default(self, key, value):
-        assert key in self.__dict__, key
-        if self.__dict__[key] is None:
-            self.__dict__[key] = value
+    def _str_conf(self, conf_v):
+        if isinstance(conf_v, (bool, str, int)):
+            return conf_v
+        if isinstance(conf_v, (
+                ipaddress.IPv4Address, ipaddress.IPv4Interface, ipaddress.IPv4Network,
+                ipaddress.IPv6Address, ipaddress.IPv6Interface, ipaddress.IPv6Network)):
+            return str(conf_v)
+        if isinstance(conf_v, (dict, OrderedDict)):
+            return {str(i): self._str_conf(j) for i, j in conf_v.items() if j is not None}
+        if isinstance(conf_v, (list, tuple, frozenset)):
+            return tuple([self._str_conf(i) for i in conf_v if i is not None])
+        if isinstance(conf_v, Conf):
+            for i in ('name', '_id'):
+                if hasattr(conf_v, i):
+                    return getattr(conf_v, i)
+        return None
 
     def to_conf(self):
         """Return configuration as a dict."""
-        return {
+        conf = {
             k: self.__dict__[str(k)] for k in self.defaults.keys() if k != 'name'}
+        return json.dumps(self._str_conf(conf), sort_keys=True, indent=4, separators=(',', ': '))
+
+    def conf_diff(self, other):
+        """Return text diff between two Confs."""
+        differ = difflib.Differ()
+        return '\n'.join(differ.compare(
+            self.to_conf().splitlines(), other.to_conf().splitlines()))
 
     def conf_hash(self, dyn=False, subconf=True, ignore_keys=None):
         """Return hash of keys configurably filtering attributes."""
@@ -177,3 +209,22 @@ class Conf:
 
     def __ne__(self, other):
         return not self.__eq__(other)
+
+    @staticmethod
+    def _check_ip_str(ip_str, ip_method=ipaddress.ip_address):
+        try:
+            # bool type is deprecated by the library ipaddress
+            if not isinstance(ip_str, bool):
+                return ip_method(ip_str)
+            else:
+                raise InvalidConfigError('Invalid IP address %s: IP address of type bool' % (ip_str))
+        except (ValueError, AttributeError, TypeError) as err:
+            raise InvalidConfigError('Invalid IP address %s: %s' % (ip_str, err))
+
+    @staticmethod
+    def _ipvs(ipas):
+        return frozenset([ipa.version for ipa in ipas])
+
+    @staticmethod
+    def _by_ipv(ipas, ipv):
+        return frozenset([ipa for ipa in ipas if ipa.version == ipv])
