@@ -54,6 +54,7 @@ class ValveFloodManager(ValveManagerBase):
         """Initialise the flood table with filtering flows."""
         ofmsgs = []
         for eth_dst, eth_dst_mask in (
+                (valve_packet.CISCO_CDP_VTP_UDLD_ADDRESS, valve_packet.mac_byte_mask(6)),
                 (valve_packet.CISCO_SPANNING_GROUP_ADDRESS, valve_packet.mac_byte_mask(6)),
                 (valve_packet.BRIDGE_GROUP_ADDRESS, valve_packet.BRIDGE_GROUP_MASK)):
             ofmsgs.append(self.flood_table.flowdrop(
@@ -72,14 +73,13 @@ class ValveFloodManager(ValveManagerBase):
     @staticmethod
     def _build_flood_local_rule_actions(vlan, exclude_unicast, in_port, exclude_all_external):
         """Return a list of flood actions to flood packets from a port."""
-        external_ports = vlan.loop_protect_external_ports_up()
+        external_ports = vlan.loop_protect_external_ports()
         exclude_ports = vlan.exclude_same_lag_member_ports(in_port)
+        exclude_ports.update(vlan.exclude_native_if_dot1x())
         if external_ports:
             if (exclude_all_external or
                     in_port is not None and in_port.loop_protect_external):
                 exclude_ports |= set(external_ports)
-            else:
-                exclude_ports |= set(external_ports[1:])
         return valve_of.flood_port_outputs(
             vlan.tagged_flood_ports(exclude_unicast),
             vlan.untagged_flood_ports(exclude_unicast),
@@ -109,50 +109,81 @@ class ValveFloodManager(ValveManagerBase):
             vlan, exclude_unicast, None)
         return (self._build_flood_rule(match, command, flood_acts, flood_priority), flood_acts)
 
-    def _build_flood_rule_for_port(self, vlan, eth_dst, eth_dst_mask, # pylint: disable=too-many-arguments
-                                   exclude_unicast, command, port,
-                                   add_match=None, preflood_acts=None,
+    @staticmethod
+    def _output_non_output_actions(flood_acts):
+        output_ports = set()
+        all_nonoutput_actions = set()
+        deduped_acts = []
+        # avoid dedupe_ofmsgs() here, as it's expensive - most of the time we are comparing
+        # port numbers as integers which is much cheaper.
+        for act in flood_acts:
+            if valve_of.is_output(act):
+                if act.port in output_ports:
+                    continue
+                output_ports.add(act.port)
+            else:
+                str_act = str(act)
+                if str_act in all_nonoutput_actions:
+                    continue
+                all_nonoutput_actions.add(str_act)
+            deduped_acts.append(act)
+        nonoutput_actions = all_nonoutput_actions - set([str(valve_of.pop_vlan())])
+        return (deduped_acts, output_ports, nonoutput_actions)
+
+    def _build_flood_acts_for_port(self, vlan, exclude_unicast, port,
                                    exclude_all_external=False):
+        flood_acts = []
+        port_output_ports = []
+        port_non_output_acts = []
+        if port.dyn_phys_up:
+            flood_acts = self._build_flood_rule_actions(
+                vlan, exclude_unicast, port, exclude_all_external)
+            flood_acts, port_output_ports, port_non_output_acts = self._output_non_output_actions(
+                flood_acts)
+        return (flood_acts, port_output_ports, port_non_output_acts)
+
+    def _build_flood_rule_for_port(self, vlan, eth_dst, eth_dst_mask, # pylint: disable=too-many-arguments
+                                   command, port, flood_acts, add_match=None):
+        ofmsgs = []
         if add_match is None:
             add_match = {}
-        if preflood_acts is None:
-            preflood_acts = []
         flood_priority = self._vlan_flood_priority(eth_dst_mask) + 1
         match = self.flood_table.match(
             vlan=vlan, in_port=port.number,
             eth_dst=eth_dst, eth_dst_mask=eth_dst_mask,
             **add_match)
-        flood_acts = preflood_acts + self._build_flood_rule_actions(
-            vlan, exclude_unicast, port, exclude_all_external)
-        return (self._build_flood_rule(match, command, flood_acts, flood_priority), flood_acts)
-
-    @staticmethod
-    def _output_ports_from_actions(flood_acts):
-        return {act.port for act in flood_acts if valve_of.is_output(act)}
+        ofmsgs = self._build_flood_rule(match, command, flood_acts, flood_priority)
+        return ofmsgs
 
     def _build_mask_flood_rules(self, vlan, eth_dst, eth_dst_mask, # pylint: disable=too-many-arguments
                                 exclude_unicast, command):
         ofmsgs = []
         if self.combinatorial_port_flood:
             for port in self._vlan_all_ports(vlan, exclude_unicast):
-                port_flood_ofmsg, _ = self._build_flood_rule_for_port(
-                    vlan, eth_dst, eth_dst_mask,
-                    exclude_unicast, command, port)
-                ofmsgs.append(port_flood_ofmsg)
+                flood_acts, port_output_ports, _ = self._build_flood_acts_for_port(
+                    vlan, exclude_unicast, port)
+                if port_output_ports:
+                    port_flood_ofmsg = self._build_flood_rule_for_port(
+                        vlan, eth_dst, eth_dst_mask, command, port, flood_acts)
+                    ofmsgs.append(port_flood_ofmsg)
         else:
             vlan_flood_ofmsg, vlan_flood_acts = self._build_flood_rule_for_vlan(
                 vlan, eth_dst, eth_dst_mask,
                 exclude_unicast, command)
             if not self.use_group_table:
                 ofmsgs.append(vlan_flood_ofmsg)
-            vlan_output_ports = self._output_ports_from_actions(vlan_flood_acts)
+            flood_acts, vlan_output_ports, vlan_non_output_acts = self._output_non_output_actions(
+                vlan_flood_acts)
             for port in self._vlan_all_ports(vlan, exclude_unicast):
-                port_flood_ofmsg, port_flood_acts = self._build_flood_rule_for_port(
-                    vlan, eth_dst, eth_dst_mask,
-                    exclude_unicast, command, port)
-                port_output_ports = self._output_ports_from_actions(port_flood_acts)
-                port_output_ports.add(port.number)
-                if vlan_output_ports != port_output_ports:
+                flood_acts, port_output_ports, port_non_output_acts = self._build_flood_acts_for_port(
+                    vlan, exclude_unicast, port)
+                if port_output_ports:
+                    port_output_ports.add(port.number)
+                    if (vlan_output_ports == port_output_ports and
+                            vlan_non_output_acts == port_non_output_acts):
+                        continue
+                    port_flood_ofmsg = self._build_flood_rule_for_port(
+                        vlan, eth_dst, eth_dst_mask, command, port, flood_acts)
                     ofmsgs.append(port_flood_ofmsg)
         return ofmsgs
 
@@ -185,8 +216,10 @@ class ValveFloodManager(ValveManagerBase):
         # actions to non unicast flooding.
         _, unicast_eth_vlan_flood_acts = self._build_flood_rule_for_vlan(
             vlan, None, None, True, command)
-        if (self._output_ports_from_actions(unicast_eth_vlan_flood_acts) !=
-                self._output_ports_from_actions(vlan_flood_acts)):
+        unicast_eth_vlan_flood_acts, unicast_output_ports, _ = self._output_non_output_actions(
+            unicast_eth_vlan_flood_acts)
+        vlan_flood_acts, vlan_output_ports, _ = self._output_non_output_actions(vlan_flood_acts)
+        if unicast_output_ports != vlan_output_ports:
             group_id += valve_of.VLAN_GROUP_OFFSET
             group = self.groups.get_entry(
                 group_id, valve_of.build_group_flood_buckets(unicast_eth_vlan_flood_acts))
@@ -263,6 +296,10 @@ class ValveFloodStackManager(ValveFloodManager):
         self.shortest_path_port = shortest_path_port
         self.dp_shortest_path_to_root = dp_shortest_path_to_root
         self._reset_peer_distances()
+        self._flood_actions_func = self._flood_actions
+        if self.stack_size == 2:
+            self._flood_actions_func = self._flood_actions_size2
+
 
     def _set_ext_flag(self, ext_flag):
         return self.flood_table.set_field(**{STACK_LOOP_PROTECT_FIELD: ext_flag})
@@ -288,9 +325,14 @@ class ValveFloodStackManager(ValveFloodManager):
 
     def _flood_actions_size2(self, in_port, external_ports,
                              away_flood_actions, toward_flood_actions, local_flood_actions):
+        if not in_port or in_port in self.stack_ports:
+            flood_prefix = []
+        else:
+            flood_prefix = self.ext_flood_not_needed if in_port.loop_protect_external else self.ext_flood_needed
+
         # Special case for stack with maximum distance 2 - we don't need to reflect off of the root.
         flood_actions = (
-            self.ext_flood_needed + away_flood_actions + local_flood_actions)
+            flood_prefix + away_flood_actions + local_flood_actions)
 
         if self._dp_is_root():
             # Default strategy is flood locally and to non-roots.
@@ -299,22 +341,22 @@ class ValveFloodStackManager(ValveFloodManager):
                 # externally, locally.
                 if external_ports:
                     flood_actions = (
-                        self.ext_flood_not_needed + away_flood_actions + local_flood_actions)
+                        flood_prefix + away_flood_actions + local_flood_actions)
         else:
             # Default strategy is flood locally and then to the root.
             flood_actions = (
-                self.ext_flood_needed + toward_flood_actions + local_flood_actions)
+                flood_prefix + toward_flood_actions + local_flood_actions)
 
             if in_port:
                 # If packet came from the root, flood it locally.
                 if in_port in self.towards_root_stack_ports:
                     flood_actions = (
-                        self.ext_flood_not_needed + local_flood_actions)
+                        flood_prefix + local_flood_actions)
                 # If we have external ports on this switch, then let the root know
                 # we have already flooded externally, locally.
                 elif external_ports:
                     flood_actions = (
-                        self.ext_flood_not_needed + toward_flood_actions + local_flood_actions)
+                        flood_prefix + toward_flood_actions + local_flood_actions)
 
         return flood_actions
 
@@ -433,7 +475,7 @@ class ValveFloodStackManager(ValveFloodManager):
         5: 1 2 3 4
         """
         exclude_ports = []
-        external_ports = vlan.loop_protect_external_ports_up()
+        external_ports = vlan.loop_protect_external_ports()
 
         if in_port and in_port in self.stack_ports:
             in_port_peer_dp = in_port.stack['dp']
@@ -446,37 +488,38 @@ class ValveFloodStackManager(ValveFloodManager):
             self.away_from_root_stack_ports, in_port, exclude_ports=exclude_ports)
         toward_flood_actions = valve_of.flood_tagged_port_outputs(
             self.towards_root_stack_ports, in_port)
-
-        if self.stack_size == 2:
-            return self._flood_actions_size2(
-                in_port, external_ports, away_flood_actions, toward_flood_actions,
-                local_flood_actions)
-        return self._flood_actions(
-            in_port, external_ports, away_flood_actions, toward_flood_actions,
-            local_flood_actions)
+        flood_acts = self._flood_actions_func(
+            in_port, external_ports, away_flood_actions,
+            toward_flood_actions, local_flood_actions)
+        return flood_acts
 
     def _build_mask_flood_rules(self, vlan, eth_dst, eth_dst_mask, # pylint: disable=too-many-arguments
                                 exclude_unicast, command):
         # Stack ports aren't in VLANs, so need special rules to cause flooding from them.
         ofmsgs = super(ValveFloodStackManager, self)._build_mask_flood_rules(
             vlan, eth_dst, eth_dst_mask, exclude_unicast, command)
-        external_ports = vlan.loop_protect_external_ports_up()
+        external_ports = vlan.loop_protect_external_ports()
         for port in self.stack_ports:
             if self.externals and external_ports:
                 # If external flag is set, flood to external ports, otherwise exclude them.
                 for ext_port_flag, exclude_all_external in (
                         (self.NONEXT_PORT_FLAG, True),
                         (self.EXT_PORT_FLAG, False)):
-                    port_flood_ofmsg, _ = self._build_flood_rule_for_port(
-                        vlan, eth_dst, eth_dst_mask,
-                        exclude_unicast, command, port,
-                        add_match={STACK_LOOP_PROTECT_FIELD: ext_port_flag},
-                        exclude_all_external=exclude_all_external)
+                    flood_acts, port_output_ports, _ = self._build_flood_acts_for_port(
+                        vlan, exclude_unicast, port, exclude_all_external=exclude_all_external)
+                    if not port_output_ports:
+                        continue
+                    port_flood_ofmsg = self._build_flood_rule_for_port(
+                        vlan, eth_dst, eth_dst_mask, command, port, flood_acts,
+                        add_match={STACK_LOOP_PROTECT_FIELD: ext_port_flag})
                     ofmsgs.append(port_flood_ofmsg)
             else:
-                port_flood_ofmsg, _ = self._build_flood_rule_for_port(
-                    vlan, eth_dst, eth_dst_mask,
-                    exclude_unicast, command, port)
+                flood_acts, port_output_ports, _ = self._build_flood_acts_for_port(
+                    vlan, exclude_unicast, port)
+                if not port_output_ports:
+                    continue
+                port_flood_ofmsg = self._build_flood_rule_for_port(
+                    vlan, eth_dst, eth_dst_mask, command, port, flood_acts)
                 ofmsgs.append(port_flood_ofmsg)
             if not self._dp_is_root():
                 # Drop bridge local traffic immediately.
