@@ -19,6 +19,7 @@
 
 from collections import defaultdict, deque
 import random
+import time
 
 import ipaddress
 
@@ -208,6 +209,21 @@ class ValveRouteManager(ValveManagerBase):
     def _vlan_nexthop_cache(self, vlan):
         return vlan.neigh_cache_by_ipv(self.IPV)
 
+    def expire_port_nexthops(self, port):
+        """Expire all hosts on a port."""
+        ofmsgs = []
+        now = time.time()
+        for vlan in port.vlans():
+            nexthop_cache = self._vlan_nexthop_cache(vlan)
+            dead_nexthops = [
+                (ip_gw, nexthop_cache_entry) for ip_gw, nexthop_cache_entry in nexthop_cache.items()
+                if nexthop_cache_entry and nexthop_cache_entry.port and
+                port.number == nexthop_cache_entry.port.number]
+            for ip_gw, nexthop_cache_entry in dead_nexthops:
+                self.logger.info('marking %s as a dead nexthop' % nexthop_cache_entry.eth_src)
+                ofmsgs.extend(self._expire_gateway_flows(ip_gw, nexthop_cache_entry, vlan, now))
+        return ofmsgs
+
     def _vlan_nexthop_cache_entry(self, vlan, ip_gw):
         nexthop_cache = self._vlan_nexthop_cache(vlan)
         return nexthop_cache.get(ip_gw, None)
