@@ -23,6 +23,7 @@ import netaddr
 
 from faucet import valve_of
 from faucet.conf import Conf, test_config_condition, InvalidConfigError
+from faucet.faucet_pipeline import STACK_LOOP_PROTECT_FIELD
 from faucet.valve_packet import FAUCET_MAC
 
 
@@ -79,7 +80,7 @@ class VLAN(Conf):
 # Note: while vlans are configured once for each datapath, there will be a
 # separate vlan object created for each datapath that the vlan appears on
 
-    mutable_attrs = frozenset(['tagged', 'untagged'])
+    mutable_attrs = frozenset(['tagged', 'untagged', 'dot1x_untagged'])
 
     defaults = {
         'name': None,
@@ -90,16 +91,6 @@ class VLAN(Conf):
         'faucet_mac': FAUCET_MAC,
         # set MAC for FAUCET VIPs on this VLAN
         'unicast_flood': True,
-        'bgp_as': None,
-        'bgp_connect_mode': 'passive',
-        'bgp_local_address': None,
-        'bgp_port': 9179,
-        'bgp_server_addresses': ['0.0.0.0', '::'],
-        'bgp_routerid': None,
-        'bgp_neighbour_addresses': [],
-        'bgp_neighbor_addresses': [],
-        'bgp_neighbour_as': None,
-        'bgp_neighbor_as': None,
         'routes': None,
         'max_hosts': 256,
         # Limit number of hosts that can be learned on a VLAN.
@@ -109,9 +100,13 @@ class VLAN(Conf):
         'proactive_nd_limit': 0,
         # Don't proactively ND for hosts if over this limit (default 2*max_hosts)
         'targeted_gw_resolution': True,
-        # If True, and a gateway has been resolved, target the first re-resolution attempt to the same port rather than flooding.
+        # If True, target the first re-resolution attempt to last known port only.
         'minimum_ip_size_check': True,
-        # If False, don't check that IP packets have a payload (must be False for OVS trace/tutorial to work)
+        # If False, don't check that IP packets have a payload (OVS trace/tutorial requires False).
+        'reserved_internal_vlan': False,
+        # If True, forward packets from the VLAN table to the VLAN_ACL table matching the VID
+        'dot1x_assigned' : False,
+        # If True, this VLAN may be dynamically added by using the Tunnel-Private-Group-ID radius attribute
         }
 
     defaults_types = {
@@ -122,16 +117,6 @@ class VLAN(Conf):
         'faucet_vips': list,
         'faucet_mac': str,
         'unicast_flood': bool,
-        'bgp_as': int,
-        'bgp_connect_mode': str,
-        'bgp_local_address': str,
-        'bgp_port': int,
-        'bgp_server_addresses': list,
-        'bgp_routerid': str,
-        'bgp_neighbour_addresses': list,
-        'bgp_neighbor_addresses': list,
-        'bgp_neighbour_as': int,
-        'bgp_neighbor_as': int,
         'routes': list,
         'max_hosts': int,
         'vid': int,
@@ -139,27 +124,22 @@ class VLAN(Conf):
         'proactive_nd_limit': int,
         'targeted_gw_resolution': bool,
         'minimum_ip_size_check': bool,
+        'reserved_internal_vlan': bool,
+        'dot1x_assigned': bool,
     }
 
     def __init__(self, _id, dp_id, conf=None):
         self.acl_in = None
         self.acls_in = None
-        self.bgp_as = None
-        self.bgp_connect_mode = None
-        self.bgp_local_address = None
-        self.bgp_neighbor_addresses = None
-        self.bgp_neighbor_as = None
-        self.bgp_neighbour_addresses = None
-        self.bgp_neighbour_as = None
-        self.bgp_port = None
-        self.bgp_routerid = None
-        self.bgp_server_addresses = None
         self.description = None
+        self.dot1x_assigned = None
+        self.dot1x_untagged = None
         self.dp_id = None
         self.faucet_mac = None
         self.faucet_vips = None
         self.max_hosts = None
         self.minimum_ip_size_check = None
+        self.reserved_internal_vlan = None
         self.name = None
         self.proactive_arp_limit = None
         self.proactive_nd_limit = None
@@ -173,9 +153,7 @@ class VLAN(Conf):
         self.acls = {}
         self.tagged = []
         self.untagged = []
-        self.bgp_server_addresses = []
-        self.bgp_neighbour_addresses = []
-        self.bgp_neighbor_addresses = []
+        self.dot1x_untagged = []
 
         self.dyn_host_cache = None
         self.dyn_host_cache_by_port = None
@@ -198,16 +176,6 @@ class VLAN(Conf):
         self._set_default('vid', self._id)
         self._set_default('name', str(self._id))
         self._set_default('faucet_vips', [])
-        self._set_default('bgp_neighbor_as', self.bgp_neighbour_as)
-        self._set_default(
-            'bgp_neighbor_addresses', self.bgp_neighbour_addresses)
-
-    @staticmethod
-    def _check_ip_str(ip_str, ip_method=ipaddress.ip_address):
-        try:
-            return ip_method(ip_str)
-        except (ValueError, AttributeError, TypeError) as err:
-            raise InvalidConfigError('Invalid IP address %s: %s' % (ip_str, err))
 
     def check_config(self):
         super(VLAN, self).check_config()
@@ -215,13 +183,15 @@ class VLAN(Conf):
         test_config_condition(not netaddr.valid_mac(self.faucet_mac), (
             'invalid MAC address %s' % self.faucet_mac))
 
-        test_config_condition(self.acl_in and self.acls_in, 'found both acl_in and acls_in, use only acls_in')
+        test_config_condition(
+            self.acl_in and self.acls_in, 'found both acl_in and acls_in, use only acls_in')
         if self.acl_in and not isinstance(self.acl_in, list):
             self.acls_in = [self.acl_in,]
             self.acl_in = None
         if self.acls_in:
             for acl in self.acls_in:
-                test_config_condition(not isinstance(acl, (int, str)), 'acl names must be int or str')
+                test_config_condition(
+                    not isinstance(acl, (int, str)), 'acl names must be int or str')
 
         if self.max_hosts:
             if not self.proactive_arp_limit:
@@ -231,32 +201,8 @@ class VLAN(Conf):
 
         if self.faucet_vips:
             self.faucet_vips = frozenset([
-                self._check_ip_str(ip_str, ip_method=ipaddress.ip_interface) for ip_str in self.faucet_vips])
-
-        if self.bgp_neighbor_addresses or self.bgp_neighbour_addresses:
-            neigh_addresses = frozenset(self.bgp_neighbor_addresses + self.bgp_neighbour_addresses)
-            self.bgp_neighbor_addresses = frozenset([
-                self._check_ip_str(ip_str) for ip_str in neigh_addresses])
-
-        if self.bgp_server_addresses:
-            self.bgp_server_addresses = frozenset([
-                self._check_ip_str(ip_str) for ip_str in self.bgp_server_addresses])
-            for ipv in self.bgp_ipvs():
-                test_config_condition(
-                    len(self.bgp_server_addresses_by_ipv(ipv)) != 1,
-                    'Only one BGP server address per IP version supported')
-
-        if self.bgp_as:
-            test_config_condition(not isinstance(self.bgp_port, int), (
-                'BGP port must be %s not %s' % (int, type(self.bgp_port))))
-            test_config_condition(self.bgp_connect_mode not in ('passive'), (
-                'BGP connect mode %s must be passive' % self.bgp_connect_mode))
-            test_config_condition(not ipaddress.IPv4Address(self.bgp_routerid), (
-                '%s is not a valid IPv4 address' % (self.bgp_routerid)))
-            test_config_condition(not self.bgp_neighbor_as, 'No BGP neighbor AS')
-            test_config_condition(not self.bgp_neighbor_addresses, 'No BGP neighbor addresses')
-            test_config_condition(len(self.bgp_neighbor_addresses) != len(self.bgp_neighbor_addresses), (
-                'Must be as many BGP neighbor addresses as BGP server addresses'))
+                self._check_ip_str(ip_str, ip_method=ipaddress.ip_interface)
+                for ip_str in self.faucet_vips])
 
         if self.routes:
             test_config_condition(not isinstance(self.routes, list), 'invalid VLAN routes format')
@@ -297,7 +243,10 @@ class VLAN(Conf):
         """Reset tagged and untagged port lists."""
         sorted_ports = sorted(ports, key=lambda i: i.number)
         self.tagged = tuple([port for port in sorted_ports if self in port.tagged_vlans])
-        self.untagged = tuple([port for port in sorted_ports if self == port.native_vlan])
+        self.untagged = tuple([port for port in sorted_ports
+                               if self == port.native_vlan and port.dyn_dot1x_native_vlan is None])
+        self.dot1x_untagged = tuple([port for port in sorted_ports
+                                     if self == port.dyn_dot1x_native_vlan])
 
     def add_cache_host(self, eth_src, port, cache_time):
         """Add/update a host to the cache on a port at at time."""
@@ -348,7 +297,6 @@ class VLAN(Conf):
     def clear_cache_hosts_on_port(self, port):
         """Clear all hosts learned on a port."""
         for entry in self.cached_hosts_on_port(port):
-
             self.expire_cache_host(entry.eth_src)
 
     def expire_cache_hosts(self, now, learn_timeout):
@@ -368,14 +316,6 @@ class VLAN(Conf):
                     [entry.cache_time for entry in self.dyn_host_cache.values()])
         return expired_hosts
 
-    @staticmethod
-    def _ipvs(ipas):
-        return frozenset([ipa.version for ipa in ipas])
-
-    @staticmethod
-    def _by_ipv(ipas, ipv):
-        return frozenset([ipa for ipa in ipas if ipa.version == ipv])
-
     def faucet_vips_by_ipv(self, ipv):
         """Return VIPs with specified IP version on this VLAN."""
         return self._by_ipv(self.faucet_vips, ipv)
@@ -387,22 +327,10 @@ class VLAN(Conf):
         other_vips = vips - link_local_vips
         return (link_local_vips, other_vips)
 
-    def bgp_neighbor_addresses_by_ipv(self, ipv):
-        """Return BGP neighbor addresses with specified IP version on this VLAN."""
-        return self._by_ipv(self.bgp_neighbor_addresses, ipv)
-
-    def bgp_server_addresses_by_ipv(self, ipv):
-        """Return BGP server addresses with specified IP version on this VLAN."""
-        return self._by_ipv(self.bgp_server_addresses, ipv)
-
     def ipvs(self):
         """Return IP versions configured on this VLAN."""
         return self._ipvs(self.faucet_vips)
 
-    def bgp_ipvs(self):
-        """Return list of IP versions for BGP configured on this VLAN."""
-        return self._ipvs(self.bgp_server_addresses)
-
     def routes_by_ipv(self, ipv):
         """Return route table for specified IP version on this VLAN."""
         return self.dyn_routes_by_ipv[ipv]
@@ -479,6 +407,8 @@ class VLAN(Conf):
             str_ports.append('tagged: %s' % ','.join([str(p) for p in self.tagged]))
         if self.untagged:
             str_ports.append('untagged: %s' % ','.join([str(p) for p in self.untagged]))
+        if self.dot1x_untagged:
+            str_ports.append('dot1x_untagged: %s' % ','.join([str(p) for p in self.dot1x_untagged]))
         return 'VLAN %s vid:%s %s' % (self.name, self.vid, ' '.join(str_ports))
 
     def __repr__(self):
@@ -486,7 +416,7 @@ class VLAN(Conf):
 
     def get_ports(self):
         """Return all ports on this VLAN."""
-        return self.tagged + self.untagged
+        return self.tagged + self.untagged + self.dot1x_untagged
 
     def hairpin_ports(self):
         """Return all ports with hairpin enabled."""
@@ -539,11 +469,22 @@ class VLAN(Conf):
             for lag, ports in lags.items():
                 ports_up = lags_up[lag]
                 if ports_up:
-                    exclude_ports.update(ports[1:])
+                    ports.remove(ports_up[0])
+                    exclude_ports.update(ports)
                 else:
                     exclude_ports.update(ports)
         return exclude_ports
 
+    def exclude_native_if_dot1x(self):
+        """Don't output on native vlan, if dynamic (1x) vlan is in use"""
+        exclude_ports = set()
+        for port in self.untagged:
+            if port.dyn_dot1x_native_vlan is None:
+                continue
+            if port.dyn_dot1x_native_vlan != self:
+                exclude_ports.add(port)
+        return exclude_ports
+
     @staticmethod
     def flood_ports(configured_ports, exclude_unicast):
         if exclude_unicast:
@@ -554,12 +495,18 @@ class VLAN(Conf):
         return self.flood_ports(self.tagged, exclude_unicast)
 
     def untagged_flood_ports(self, exclude_unicast):
-        return self.flood_ports(self.untagged, exclude_unicast)
+        return self.flood_ports(self.untagged + self.dot1x_untagged, exclude_unicast)
 
-    def output_port(self, port, hairpin=False):
-        actions = port.mirror_actions()
+    def output_port(self, port, hairpin=False, output_table=None, loop_protect_field=None):
+        actions = []
         if self.port_is_untagged(port):
             actions.append(valve_of.pop_vlan())
+            # Packet is mirrored, as the receiving host sees it (without a tag).
+            actions.extend(port.mirror_actions())
+        else:
+            actions.extend(port.mirror_actions())
+            if loop_protect_field is not None:
+                actions.append(output_table.set_field(**{STACK_LOOP_PROTECT_FIELD: loop_protect_field}))
         if hairpin:
             actions.append(valve_of.output_port(valve_of.OFP_IN_PORT))
         else:
@@ -581,16 +528,15 @@ class VLAN(Conf):
             if ports:
                 pkt = packet_builder(vid, *args)
                 exclude_ports = self.exclude_same_lag_member_ports()
-                running_ports = [
-                    port for port in ports if port.running() and port not in exclude_ports]
-                random.shuffle(running_ports)
-                if multi_out:
-                    ofmsgs.append(valve_of.packetouts(
-                        [port.number for port in running_ports], pkt.data))
-                else:
-                    ofmsgs.extend(
-                        [valve_of.packetout(port.number, pkt.data)
-                         for port in running_ports])
+                running_port_nos = [
+                    port.number for port in ports if port.running() and port not in exclude_ports]
+                if running_port_nos:
+                    random.shuffle(running_port_nos)
+                    if multi_out:
+                        ofmsgs.append(valve_of.packetouts(running_port_nos, pkt.data))
+                    else:
+                        ofmsgs.extend(
+                            [valve_of.packetout(port_no, pkt.data) for port_no in running_port_nos])
         return ofmsgs
 
     def port_is_tagged(self, port):
@@ -599,7 +545,7 @@ class VLAN(Conf):
 
     def port_is_untagged(self, port):
         """Return True if port number is an untagged port on this VLAN."""
-        return port in self.untagged
+        return port in self.untagged or port in self.dot1x_untagged
 
     def vip_map(self, ipa):
         for faucet_vip in self.faucet_vips:
@@ -636,20 +582,3 @@ class VLAN(Conf):
         if self.is_faucet_vip(dst_ip) and self.ip_in_vip_subnet(src_ip):
             return True
         return False
-
-    def to_conf(self):
-        result = super(VLAN, self).to_conf()
-        if result is not None:
-            if self.routes:
-                result['routes'] = [{'route': route} for route in self.routes]
-            if self.faucet_vips:
-                result['faucet_vips'] = [str(vip) for vip in self.faucet_vips]
-            if self.bgp_neighbor_addresses:
-                result['bgp_neighbor_addresses'] = [str(vip) for vip in self.bgp_neighbor_addresses]
-            if self.bgp_server_addresses:
-                result['bgp_server_addresses'] = [str(vip) for vip in self.bgp_server_addresses]
-            if 'bgp_neighbor_as' in result:
-                del result['bgp_neighbor_as']
-            if 'bgp_neighbor_addresses' in result:
-                del result['bgp_neighbor_addresses']
-        return result
