@@ -1,12 +1,11 @@
 #!/bin/bash
 
-PYV="3.5"
 FAUCETHOME=`dirname $0`"/../.."
 TMPDIR=`mktemp -d -p /var/tmp`
 CONFIG="$FAUCETHOME/setup.cfg"
 PARARGS="parallel --delay 1 --bar --halt now,fail=1 -j 2"
 PYTYPE=`which pytype`
-PYTYPEARGS="python$PYV $PYTYPE --config $CONFIG -o $TMPDIR/{/} {}"
+PYTYPEARGS="python3 $PYTYPE --config $CONFIG -o $TMPDIR/{/} {}"
 PYHEADER=`head -1 $PYTYPE`
 SRCFILES="$FAUCETHOME/tests/codecheck/src_files.sh $*"
 echo "Using $PYTYPE (header $PYHEADER)"
