@@ -4,10 +4,11 @@
 
 import logging
 import sys
+
+import afl
 from ryu.controller import dpset
 from faucet import faucet
 from faucet import faucet_experimental_api
-import afl
 import fake_packet
 
 ROUNDS = 1
@@ -29,12 +30,12 @@ def main():
             valve.dp.running = True
             valve.dp.dyn_finalized = state
 
-    while afl.loop(ROUNDS):
+    while afl.loop(ROUNDS): # pylint: disable=c-extension-no-member
         # receive input from afl
         rcv = sys.stdin.read()
         data = None
         try:
-            data = bytearray.fromhex(rcv) # pytype: disable=missing-parameter
+            data = bytearray.fromhex(rcv) # pytype: disable=missing-parameter, disable=wrong-arg-types
         except (ValueError, TypeError):
             continue
 
